#include <iostream>
#include "AlberoBinario.h"

using namespace std;

void initAlbero(Albero& a) {
	a = NULL;
}

void createRoot(Albero& a, int info) {
NodoAlbero* root = new NodoAlbero();
	root->father = NULL;
	root->left = NULL;
	root->right= NULL;
	root->info = info;
	a = root;
}

int isEmpty(Albero& a) {
	return (a == NULL);
}

NodoAlbero* getFather(NodoAlbero* x) {
	return x->father;
}

NodoAlbero* getLeft(NodoAlbero* x) {
	return x->left;
}

NodoAlbero* getRight(NodoAlbero* x) {
	return x->right;
}


NodoAlbero* left_right_insert(NodoAlbero* x, int info, bool left) {
	NodoAlbero* node = new NodoAlbero();
	node->father = x;
	node->left = NULL;
	node->right = NULL;
	node->info = info;
	if (left) {
		if (x->left !=NULL)
			deallocaAlbero(x->left);
		x->left = node;
	}
	else{
		if (x->right !=NULL)
			deallocaAlbero(x->right);
		x->right = node;
	}
	return node;
}



NodoAlbero* insert(NodoAlbero* x, int info) {
	if (x->right==NULL) {
		NodoAlbero* node = new NodoAlbero();
		node->father = x;
		node->left = NULL;
		node->right = NULL;
		node->info = info;
		x->right = node;
		return node;
	}
	else if (x->left==NULL) {
		NodoAlbero* node = new NodoAlbero();
		node->father = x;
		node->left = NULL;
		node->right = NULL;
		node->info = info;
		x->left = node;
		return node;
	}
	else
		return NULL;
}

void BST_insert (Albero &r, int info){
	/* Il nuovo elemento viene inserito come foglia del BST */
	NodoAlbero* new_leaf = new NodoAlbero();
	new_leaf->left = NULL;
	new_leaf->right = NULL;
	new_leaf->info = info;

	/* Il padre di neaw_leaf deve essere ricercato all'interno del BST */
	NodoAlbero * current_node = r;
	NodoAlbero * current_node_father = NULL;
	while (current_node!=NULL) {
		current_node_father = current_node;
		if (info < current_node->info)
			current_node = current_node->left;
		else
			current_node = current_node->right;
	}
	new_leaf->father = current_node_father;
	if (current_node_father == NULL) /* new_leaf  il primo nodo inserito nel BST e ne diventa la radice */
		r = new_leaf;
	else
		if (info < current_node_father->info)
			current_node_father->left = new_leaf;
		else
			current_node_father->right = new_leaf;
}


void BST_del(Albero &r, int info){
	NodoAlbero* nodo_da_rimuovere = cerca_nodo(r,info);
	if (nodo_da_rimuovere==NULL)
		return;
	
	NodoAlbero* nodo;
	/* nodo_da_rimuovere manca di almeno un figlio? */
	if ( (nodo_da_rimuovere->left==NULL) || (nodo_da_rimuovere->right==NULL) )
		/* nodo coincide con nodo_da_rimuovere */
		nodo = nodo_da_rimuovere; 
	else
		  /* nodo coincide con il successore di nodo_da_rimuovere, 
		  che (per definizione) ha al pi un figlio destro */
		  nodo = successore (nodo_da_rimuovere);
	/* A questo punto, il nodo da rimuovere  nodo con al pi un solo figlio. 
	   Pertanto si recupera l'eventuale "figlio" di "nodo", per poi collegarlo 
	   come figlio del padre di nodo*/
	NodoAlbero* figlio;
	if (nodo->left!=NULL)
		figlio = nodo->left;
	else 
		figlio = nodo->right;
	/* Si aggiorna il padre di "figlio" */
	if (figlio != NULL)
		figlio->father = nodo->father;
	/* nodo  la radice dell'albero? Se si, x deve divenire la nuova radice */
	if (nodo->father == NULL)
		r = figlio;
	/* Se nodo non  la radice dell'albero, bisogna collegare figlio al padre di nodo*/
	else 
		if (nodo == nodo->father->left)
			nodo->father->left = figlio;
		else
			nodo->father->right = figlio;
	/* Infine, nel caso in cui nodo_da_rimuovere possegga entrambi i figli rendendo cos 
	   necessaria l'individuazione del suo successore, bisogna ricopiare le informazioni 
	   del successore nel nodo da eliminare*/
	if (nodo != nodo_da_rimuovere)
		nodo_da_rimuovere->info = nodo->info;
	/* Rimozione del nodo */
	delete(nodo);
}

void del(NodoAlbero* x) {
	if (x->left==NULL && x->right==NULL) {
		(x->father)->left = NULL;
		(x->father)->right =NULL;
		delete(x);
	}
}

void deallocaAlbero(Albero& a) {
	if( a!=NULL ) {
    		deallocaAlbero(a->left);
    		deallocaAlbero(a->right);
  	}
  	delete(a);
}

NodoAlbero* min(NodoAlbero* r){
	NodoAlbero* current_node = r;
	while (current_node->left != NULL)
		current_node = current_node->left;
	return current_node;
}

NodoAlbero* max(NodoAlbero* r){
	NodoAlbero* current_node = r;
	while (current_node->right != NULL)
		current_node = current_node->right;
	return current_node;
}

NodoAlbero* successore(NodoAlbero* n){
	NodoAlbero* current_node = n;
	if (current_node->right != NULL)
		return min(current_node->right);
	NodoAlbero* current_node_father = current_node->father;
	while ( (current_node_father != NULL) && (current_node == current_node_father->right) ){
		current_node = current_node_father;
		current_node_father = current_node_father->father;
	}
	return current_node_father;
}

NodoAlbero* predecessore(NodoAlbero* n){
	NodoAlbero* current_node = n;
	if (current_node->left != NULL)
		return max(current_node->left);
	NodoAlbero* current_node_father = current_node->father;
	while ( (current_node_father != NULL) && (current_node == current_node_father->left) ){
		current_node = current_node_father;
		current_node_father = current_node_father->father;
	}
	return current_node_father;
}


NodoAlbero* cerca_nodo(NodoAlbero* a,int info){
	if ( (a == NULL) || (a->info == info) )
		return a;
	if (info < a->info)
		return cerca_nodo(a->left,info);
	else
		return cerca_nodo(a->right,info);
}


int conta_foglie(Albero &r) {
	// caso base (albero vuoto)
	if (r == NULL) return 0;
	// caso ricorsivo (albero non vuoto)
	else
	{
		if( (r->left == NULL) && (r->right == NULL) )
			return 1;
		else
			return (conta_foglie(r->left) + conta_foglie(r->right));
	}
}

int conta_nodi_interni (Albero& r){
	// caso base (albero vuoto)
	if (r == NULL) return 0;
	// caso ricorsivo (albero non vuoto)
	else
	{
		if ( (r->left != NULL) || (r->right != NULL) )
			return(1 + conta_nodi_interni(r->left) + conta_nodi_interni(r->right));
		else
			return 0;
	}
}


int bilanciato (Albero r,int & nodi){
	/* Condizione d'arresto: l'albero vuoto  bilanciato */
	if (r == NULL) { 
		  nodi = 0; 
		  return 1;
	}
	/* L'albero con radice r e sottoalberi Bs e Bd  bilanciato 
	   se Bs e Bd sono bilanciati e hanno lo stesso numero di nodi.*/
	int nodi_s , nodi_d ;
	int bal_s = bilanciato (r->left,nodi_s);
	int bal_d = bilanciato (r->right,nodi_d);
	nodi = nodi_s + nodi_d +1;
	
	int max,min;
	if (nodi_s >= nodi_d){
		max = nodi_s;
		min = nodi_d;
	}
	else {
		max = nodi_d;
		min = nodi_s;
	}

	return (bal_s && bal_d && (max-min<=1));
}

int profondit(Albero r){
	/*La profondit dell'albero vuoto vale 0.*/
	if (r== NULL)
		return 0;
	/*La profondit di un albero non vuoto  il massimo tra le profondit dei sottalberi destro e sinistro, pi uno.*/
    int profondita_sottoalbero_sinistro=0;
	int profondita_sottoalbero_destro=0;
	/* Calcola la profondita' dei sottoalberi */
	
    profondita_sottoalbero_sinistro = profondit(r->left);
	profondita_sottoalbero_destro = profondit(r->right);
    if (profondita_sottoalbero_sinistro >= profondita_sottoalbero_destro)
		return profondita_sottoalbero_sinistro+1;
	else
		return profondita_sottoalbero_destro+1;
}
