#ifndef ALBEROBINARIO_H
#define ALBEROBINARIO_H

using namespace std;

struct NodoAlbero 
{
	int info;
	NodoAlbero *left, *right, *father;
};

typedef NodoAlbero * Albero;

/* Inizializza l'albero vuoto */
void initAlbero(Albero& a);

/* y diventa radice dell'albero se questo  inizialmente vuoto */
void createRoot(Albero& a, int info);

/* verifica se l'albero  vuoto o meno */
int isEmpty(Albero& a);

/* restituisce il padre di x */
NodoAlbero* getFather(NodoAlbero* x);

/* restituisce figlio sinistro di x */
NodoAlbero* getLeft(NodoAlbero* x);

/* restituisce figlio destro di x */
NodoAlbero* getRight(NodoAlbero* x);

/* inserisce y come figlio di x, prova ad inserirlo prima
   come figlio dx, se  gi presente come figlio sx, 
   se  gi presente anche questo non fa nulla, 
   restituisce il puntatore al nodo inserito, NULL se non 
   viene inserito nulla */
NodoAlbero* insert(NodoAlbero* x, int info);

/* cancella un nodo solo se x  foglia */
void del(NodoAlbero* x);

/* dealloca la memoria occupata da un albero */
void deallocaAlbero(Albero& a);

/* Inserimento di info all'interno di un BST che preserva la relazione d'ordine sugli interi */
void BST_insert (Albero &r, int info);

/* Rimozione del nodo contenente il valore info all'interno di un BST. La relazione d'ordine sugli interi del BST  preservata*/
void BST_del(Albero &r, int info);

/* Trova il nodo con il valore minimo del campo info all'interno di un BST */
NodoAlbero* min(NodoAlbero* r);

/* Trova il nodo con il valore massimo del campo info all'interno di un BST */
NodoAlbero* max(NodoAlbero* r);

/* Cerca il successore del nodo n all'interno di un BST */
NodoAlbero* successore(NodoAlbero* n);

/* Cerca il predecessaore del nodo n all'interno di un BST */
NodoAlbero* predecessore(NodoAlbero* n);

/* Cerca il nodo contenente il valore info all'interno di un BST */
NodoAlbero* cerca_nodo(NodoAlbero* a,int info);

/* Conta il numero di foglie in un albero binario*/
int conta_foglie(Albero &r);

/* Conta il numero di nodi interni di un albero binario*/
int conta_nodi_interni (Albero& r);

/* Calcola la profondit di un albero binario */
int profondit(Albero r);

/* Restituisce 1 se l'albero binario r  bilanciato, 0 altrimenti */
int bilanciato(Albero r,int & nodi);

/* Inserisce un nuovo nodo con valore info come figlio sinistro di x se left  true. 
Altrimenti, info  inserito come figlio destro */
NodoAlbero* left_right_insert(NodoAlbero* x, int info, bool left); 

#endif
