#include <iostream>
#include "Coda.h"

using namespace std;


/* 
visita in preordine: 
          si analizza la radice, 
          poi si visita in preordine il sottoalbero sinistro, 
          quindi si visita in preordine il sottoalbero destro 
*/
void VisitaAlberoProfondita_PreOrdine(Albero& a) {
	if (a == NULL)
		return;
	cout << a->info << '\t';
	VisitaAlberoProfondita_PreOrdine(a->left);
	VisitaAlberoProfondita_PreOrdine(a->right);
}


/*
visita in postordine: 
       si visita in postordine il sottoalbero sinistro, 
       poi si visita in postordine il sottoalbero destro, 
       infine si analizza la radice
*/
void VisitaAlberoProfondita_PostOrdine(Albero& a) {
	if (a == NULL)
		return;
	VisitaAlberoProfondita_PostOrdine(a->left);
	VisitaAlberoProfondita_PostOrdine(a->right);
	cout << a->info << '\t';
}

/*
visita in ordine simmetrico (inorder): 
       si visita in ordine simmetrico il sottoalbero sinistro, 
       poi si analizza la radice, 
       infine si visita in ordine simmetrico il sottoalbero destro
*/
void VisitaAlberoProfondita_InOrdine(Albero& a) {
	if (a == NULL)
		return;
	VisitaAlberoProfondita_InOrdine(a->left);
	cout << a->info << '\t';
	VisitaAlberoProfondita_InOrdine(a->right);
}

void VisitaAlberoAmpiezza(Albero& a) {
	Coda c;
	InitCoda(c);
	NodoAlbero* nr = getRight(a);
	NodoAlbero* nl = getLeft(a);
	
	cout << endl;
	cout << a->info << '\t';;

	if (nr!=NULL)
		InsertCoda(c, *nr);
	if (nr!=NULL)
		InsertCoda(c, *nl);
		
	while (!EmptyCoda(c)) {
		NodoAlbero n;
		ExtractCoda(c, n);
		
		cout << n.info << '\t';
		
		NodoAlbero* nnr = getRight(&n);
		NodoAlbero* nnl = getLeft(&n);
		
		if (nnr!=NULL)
			InsertCoda(c, *nnr);
		if (nnl!=NULL)
			InsertCoda(c, *nnl);
	}
	
}




/* La seguente funzione risolve il problema descritto di seguito
		Scrivere un algoritmo ricorsivo che riceve in input un AlberoBinario e verifica se ogni nodo foglia contiene un intero pari.
*/
void Visita_PreOrdine_Per_Foglie(Albero& a, bool& trovato) {
	if (a == NULL)
		return;
	/* Se il nodo  una foglia */
	if ( (a->left==NULL) && (a->right == NULL) )
		/* Se la foglia non contiene un valore pari */
		if (a->info % 2 != 0){
			trovato = true;
			cout <<" Il nodo "<<a->info<<" non contiene un numero pari"<<endl;
		}
	if (!trovato)
		Visita_PreOrdine_Per_Foglie(a->left, trovato);
	if (!trovato) 
		Visita_PreOrdine_Per_Foglie(a->right, trovato);

}



/* La seguente funzione risolve il problema descritto di seguito
		Scrivere un algoritmo ricorsivo che riceve in input un AlberoBinario e verifica se esiste in esso almeno un nodo n tale che:
			a. n abbia entrambi i figli;
			b. la parte informativa del figlio sinistro e del figlio destro di n  uguale a quella di  n.
*/

void Visita_PreOrdine_Confronto_Padre_Figli(Albero& a, bool & trovato) {
	if (a == NULL)
		return;
	/*Se il nodo a possiede entrambi i figli*/
	if ( (a->left != NULL) && (a->right != NULL) )
		/* Se i figli hanno lo stesso contenuto informativo */
		if (a->left->info == a->right->info)
			/* Se i figli hanno lo stesso contenuto informativo del padre */
			if (a->info == a->left->info) {
				trovato = true;
				cout << "Nodo trovato: "<<a->info<<endl;
			}
	if (!trovato)
		Visita_PreOrdine_Confronto_Padre_Figli(a->left,trovato);
	if (!trovato) 
		Visita_PreOrdine_Confronto_Padre_Figli(a->right,trovato);

}



/* Le due seguenti funzioni sono la risoluzione del seguente quesito:
   
		Scrivere un algoritmo ricorsivo che riceve in input un AlberoBinario e verifica se esiste in esso almeno un nodo n tale che:
			- la somma dei valori pari contenuti nei nodi del sottoalbero sinistro di n  uguale alla somma dei valori dispari contenuti 
			  nei nodi del sottoalbero destro di n.
*/
void somma_sottoalbero (NodoAlbero* nodo, bool pari, int & somma) {
	if (nodo == NULL)
		return;
	if ( ( (pari) && (nodo->info%2==0) ) || ( (!pari) && (nodo->info%2!=0) ) )
		somma += nodo->info;
	somma_sottoalbero (nodo->left, pari, somma);
	somma_sottoalbero (nodo->right, pari, somma);
}


void Visita_PreOrdine_Per_Somma(Albero& a, bool & trovato) {
	if (a == NULL)
		return;
	int somma_s = 0;
	somma_sottoalbero(a->left, true, somma_s);
	int somma_d = 0;
	somma_sottoalbero(a->right, false, somma_d);
	/*cout<<"Nodo corrente: "<<a->info<<endl;
	cout<<"Somma sinistra: "<<somma_s<<endl;
	cout<<"Somma destra: "<<somma_d<<endl<<endl<<endl;*/
	
	if (somma_s == somma_d) {
		/* Evita il caso somma_s == somma_d == 0*/
		if (somma_s != 0){
			trovato =true;
			cout << "Nodo trovato: " <<a->info << endl;
		}
	}
	if (!trovato) 
		Visita_PreOrdine_Per_Somma(a->left,trovato);
	if (!trovato)
		Visita_PreOrdine_Per_Somma(a->right,trovato);

}


int main() {
	Albero a;
	//NodoAlbero na;
	
	initAlbero(a);
	createRoot(a, 3);
	NodoAlbero* n1 = left_right_insert(a,2,true);
	NodoAlbero* n2 = left_right_insert(a,7,false);
	NodoAlbero* n3 = left_right_insert(n1,4,true);
	NodoAlbero* n4 = left_right_insert(n1,1,false);
	NodoAlbero* n5 = left_right_insert(n2,2,true);
	//NodoAlbero* n6 = left_right_insert(n2,7,false);
	//NodoAlbero* n6 = left_right_insert(n2,5,false);
	//NodoAlbero* n7 = left_right_insert(n3,6,true);
	NodoAlbero* n8 = left_right_insert(n4,2,false);
	
	//NodoAlbero* n9 = left_right_insert(n6,7,false);
	//NodoAlbero* n10 = left_right_insert(n5,10,false);

	/*cout<<"Visita in preordine: "<<endl;
    VisitaAlberoProfondita_PreOrdine(a);
	cout<<endl;
	cout<<"Visita in postordine: "<<endl;
	VisitaAlberoProfondita_PostOrdine(a);
	cout<<endl;
	cout<<"Visita simmetrica: "<<endl;
	VisitaAlberoProfondita_InOrdine(a);
	cout<<endl;
	cout<<"Visita in ampiezza: "<<endl;
	VisitaAlberoAmpiezza(a);
	cout<<endl;*/
	
	//NodoAlbero* nr = Right(n1);
	//cout << endl << nr->info;
    
	VisitaAlberoProfondita_PreOrdine(a);
	cout<<endl;

	bool trovato=false;
	cout<<"--------------------------------"<<endl;
	//Visita_PreOrdine_Per_Somma(a,trovato);
	//Visita_PreOrdine_Confronto_Padre_Figli(a,trovato);
	//Visita_PreOrdine_Per_Foglie(a,trovato);
	//cout<<"Lungheza media dei cammini: "<<Lunghezza_media_cammini(a)<<endl;
	cout<<"--------------------------------"<<endl;

	cout<<"BST"<<endl<<endl;

	Albero bst;
	initAlbero(bst);
	BST_insert(bst,6);
	BST_insert(bst,3);
	BST_insert(bst,1);
	BST_insert(bst,0);
	BST_insert(bst,2);
	BST_insert(bst,5);
	BST_insert(bst,4);
	BST_insert(bst,8);
	BST_insert(bst,7);
	BST_insert(bst,9);
	//BST_insert(bst,10);
	//BST_insert(bst,11);
	/*NodoAlbero* r = cerca_nodo(bst,6);
	cout<<endl<<"Altezza: "<<profondit(bst)<<endl;
	cout<<endl<<"Numero foglie: "<<conta_foglie(bst)<<endl;
	cout<<endl<<"Numero nodi interni: "<<conta_nodi_interni(bst)<<endl;
	cout<<endl<<"Min: "<<min(bst)->info<<endl;
	cout<<endl<<"Max: "<<max(bst)->info<<endl;
	cout<<endl<<"Predecessore di 3: ";
	NodoAlbero * n = cerca_nodo(r,3);
	NodoAlbero * pred = predecessore(n);
	cout<<pred->info<<endl;

	cout<<endl<<"Successore di 3: ";
	n = cerca_nodo(r,3);
	NodoAlbero * succ = successore(n);
	cout<<succ->info<<endl;*/

	
	VisitaAlberoProfondita_InOrdine(bst);
	cout<<endl<<"Visita per livelli del BST:"<<endl;
	VisitaAlberoAmpiezza(bst);

	//cout<<endl<<"Rimozione dell'elemento 9 dal BST:"<<endl;
	//BST_del(bst,9);
	//VisitaAlberoAmpiezza(bst);

	//cout<<endl<<"Rimozione dell'elemento 7 dal BST:"<<endl;
	//BST_del(bst,7);
	//VisitaAlberoAmpiezza(bst);

	cout<<endl<<"Rimozione dell'elemento 0 dal BST:"<<endl;
	BST_del(bst,0);
	VisitaAlberoAmpiezza(bst);

	cout<<endl<<"Rimozione dell'elemento 2 dal BST:"<<endl;
	BST_del(bst,2);
	VisitaAlberoAmpiezza(bst);

	cout<<endl<<"Rimozione dell'elemento 4 dal BST:"<<endl;
	BST_del(bst,4);
	VisitaAlberoAmpiezza(bst);
	
	int i;
	cout<<endl<<"Bilanciato:"<<bilanciato(bst,i)<<endl;

	/*cout<<endl<<"Rimozione dell'elemento 3 dal BST:"<<endl;
	BST_del(bst,3);
	VisitaAlberoAmpiezza(bst);

	cout<<endl<<"Rimozione dell'elemento 6 dal BST:"<<endl;
	BST_del(bst,6);
	VisitaAlberoAmpiezza(bst);

	cout<<endl<<"Numero foglie: "<<conta_foglie(bst)<<endl;*/

	cout<<endl;

    system("pause");
	return 0;
	
}
